
Imports System.Collections
Imports System.Windows.Forms




'Purpose: Sorts listviewitems by column
'
'
'Note: Modified from: http://support.microsoft.com/kb/319399/EN-US/
'
'

'IMPLEMENTATION:
'Paste the following code into the ColumnClick event for the ListView:

'' Determine if the clicked column is already the column that is
''If (e.Column = lvwColumnSorter.SortColumn) Then
'' Reverse the current sort direction for this column.
'    If (lvwColumnSorter.Order = SortOrder.Ascending) Then
'        lvwColumnSorter.Order = SortOrder.Descending
'    Else
'        lvwColumnSorter.Order = SortOrder.Ascending
'    End If
'Else
'' Set the column number that is to be sorted; default to ascending.
'    lvwColumnSorter.SortColumn = e.Column
'    lvwColumnSorter.Order = SortOrder.Ascending
'End If

'' Perform the sort with these new sort options.
'Me.ListView1.Sort()

'Note: Use Checkfieldtype method if you want to sort a number/date field

Public Class LVWColSorter
    Implements System.Collections.IComparer


    Public Enum DType As Integer

        DontCare = 0
        IntType = 1
        DecType = 2
        DateType = 4
        StringType = 8

    End Enum


#Region "Class Variables"

    Private ColumnToSort As Integer
    Private OrderOfSort As SortOrder
    Private ObjectCompare As CaseInsensitiveComparer
    Public FieldDataType As DType = DType.StringType

#End Region


#Region "Properties"

    Public Property SortColumn() As Integer
        Set(ByVal Value As Integer)
            ColumnToSort = Value
        End Set

        Get
            Return ColumnToSort
        End Get
    End Property

    Public Property Order() As SortOrder
        Set(ByVal Value As SortOrder)
            OrderOfSort = Value
        End Set

        Get
            Return OrderOfSort
        End Get
    End Property

#End Region


#Region "Constructors"

    Public Sub New()
        ' Initialize the column to '0'.
        ColumnToSort = 0

        ' Initialize the sort order to 'none'.
        OrderOfSort = SortOrder.None

        ' Initialize the CaseInsensitiveComparer object.
        ObjectCompare = New CaseInsensitiveComparer
    End Sub

    'Purpose: Instantiates to a fielddata type
    Public Sub New(ByRef rLVW As ListView, ByVal vSortColumn As Integer, Optional ByVal vFieldDataType As DType = DType.DontCare)

        Call Me.New()

        If vFieldDataType = DType.DontCare Then
            'Determine the datafield of the type
            Me.FieldDataType = Me.CheckFieldType(rLVW, vSortColumn)
        Else
            'They specfieid a datatype
            Me.FieldDataType = vFieldDataType
        End If

    End Sub

#End Region


#Region "Methods"

    'Purpose: This actually performs the comparision between to listview items
    Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements IComparer.Compare
        Dim compareResult As Integer
        Dim listviewX As ListViewItem
        Dim listviewY As ListViewItem

        ' Cast the objects to be compared to ListViewItem objects.
        listviewX = CType(x, ListViewItem)
        listviewY = CType(y, ListViewItem)


        If FieldDataType = DType.StringType Then
            'String

            ' Compare the two items.
            compareResult = ObjectCompare.Compare(listviewX.SubItems(ColumnToSort).Text, listviewY.SubItems(ColumnToSort).Text)

        ElseIf FieldDataType = DType.DateType Then

            'compareresult = DateDiff(DateInterval.Day, CDate(lv1.SubItems(mColumn).Text), CDate(lv2.SubItems(mColumn).Text))

            'Try
            '    compareResult = DateDiff(DateInterval.Day, Convert.ToDateTime(listviewX.SubItems(ColumnToSort).Text), Convert.ToDateTime(listviewY.SubItems(ColumnToSort).Text))
            'Catch ex As Exception
            '    compareResult = -1
            'End Try

            Dim tDateX As Date
            Dim tDateY As Date

            Try
                tDateX = Convert.ToDateTime(listviewX.SubItems(ColumnToSort).Text)
            Catch ex As Exception
                tDateX = #1/1/1900#
            End Try

            Try
                tDateY = Convert.ToDateTime(listviewY.SubItems(ColumnToSort).Text)
            Catch ex As Exception
                tDateY = #1/1/1900#
            End Try


            compareResult = DateDiff(DateInterval.Day, tDateX, tDateY)


        ElseIf FieldDataType = DType.DecType Then
            'Numbers


            'Note: Might have a conversion error 
            Try



                If CDec(listviewX.SubItems(ColumnToSort).Text) >= CDec(listviewY.SubItems(ColumnToSort).Text) Then
                    compareResult = 1
                Else
                    compareResult = -1
                End If

            Catch ex As Exception
                'There must have been a conversion error
                compareResult = -1
            End Try





        End If

        ' Calculate the correct return value based on the object
        ' comparison.
        If (OrderOfSort = SortOrder.Ascending) Then
            ' Ascending sort is selected, return typical result of
            ' compare operation.
            Return compareResult
        ElseIf (OrderOfSort = SortOrder.Descending) Then
            ' Descending sort is selected, return negative result of
            ' compare operation.
            Return (-compareResult)
        Else
            ' Return '0' to indicate that they are equal.
            Return 0
        End If
    End Function

#End Region


    'Purpose: Determines the type of field used for sorting on
    '
    'Note: Uses the first item on the list to determine the data type. Could modify though
    '
    'Note: Sometimes there might be data missing or null data this will create
    '      issues based on the first item on the list and create conversion
    '      issues.
    '
    'Note: You do not nessecerily have to use this to determine the field type because on
    '      mixed fields values, like strings and numbers 
    '
    'Note: Might want to look into grabbing more than on item on the list because if there
    '      is a sort then the nulled field will go to the top however the next the null field
    '      will be used to check the type and default to string
    '
    'Note: If you want to be 100% you are sorting on the right datatype then specify the FieldDataType
    '      manually.

    Public Shared Function CheckFieldType(ByRef rLVW As ListView, ByVal vSortColumn As Integer) As DType

        Dim tItem As ListViewItem
        Dim tItemB As ListViewItem 'Another listviewitem to compare against
        Dim tItemC As ListViewItem 'Another item to compare against

        Dim RandGen As New Random(DateTime.Now.Ticks Mod System.Int32.MaxValue)

        Dim RandNumber As Integer



        'If they don't have items or have only one item then it really
        'doesn't matter the datatype because you cant sort
        If rLVW.Items.Count <= 1 Then Return DType.StringType


        tItem = rLVW.Items(0)

        'Grabbing a random listview item to check for the field data as well
        '
        'Note: RandGen.Next will only produce 1 for 1,1 and 1,2, it appears the upper
        '      bound isn't a possiblity.

        tItemB = rLVW.Items(RandGen.Next(1, rLVW.Items.Count - 1))

        tItemC = rLVW.Items(RandGen.Next(1, rLVW.Items.Count - 1))

        If IsNumeric(tItem.SubItems(vSortColumn).Text) = True Or IsNumeric(tItemB.SubItems(vSortColumn).Text) = True Or IsNumeric(tItemC.SubItems(vSortColumn).Text) = True Then
            'Number

            Return DType.DecType

            'Note: This creates a massive time hit because isdate actually
            '      uses DateTime.Parse and catches an exception so the first time
            '      IsDate is errors the computer will hang until it loads up the
            '      exception stuff in memory. 

        ElseIf IsDate(tItem.SubItems(vSortColumn).Text) = True Or IsDate(tItemB.SubItems(vSortColumn).Text) = True Or IsDate(tItemC.SubItems(vSortColumn).Text) = True Then
            'Date
            Return DType.DateType
        Else
            'Essentially defaulting to string
            Return DType.StringType
        End If

    End Function


End Class

