
Imports System.Data


''' <summary>
''' Represents a SQL Query built statement
''' </summary>
''' <remarks></remarks>
Public Class SQLQueryBuild


    Public QItems As Generic.List(Of SQLQueryItem)

    Private m_SQLBase As String = ""

    Public OrderBY As String


#Region " Properties "

    Public Property SQLBase() As String
        Get
            Return Me.m_SQLBase
        End Get
        Set(ByVal value As String)
            Me.m_SQLBase = value
        End Set
    End Property

#End Region


#Region "Constructors "

    Public Sub New()

        QItems = New Generic.List(Of SQLQueryItem)

    End Sub

    Public Sub New(ByVal vSQLBase As String)

        Me.New()
        Me.m_SQLBase = vSQLBase

    End Sub

#End Region



    ''' <summary>
    ''' Retrieves the order by portion of the sql statement
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property OrderByOut() As String
        Get
            If Me.OrderBY = "" Then
                Return ""
            Else
                Return " ORDER BY " & Me.OrderBY
            End If
        End Get
    End Property

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function SQLOut() As String

        Dim SQL As String = ""
        Dim Counter1 As Integer

        For Each tItem As SQLQueryItem In QItems
            Counter1 += 1

            If Counter1 = 1 Then
                SQL = SQL & tItem.SQLOut
            Else
                'SQL = SQL & tItem.SQLOut & " AND "
                SQL = SQL & " " & tItem.CombinerStr & " " & tItem.SQLOut
            End If


        Next


        If SQL = "" Then
            'They didn't have an variables
            SQL = SQLBase & Me.OrderByOut

        Else

            'removing the last and
            SQL = SQLBase & " WHERE " & SQL & Me.OrderByOut


        End If

        Return SQL

    End Function

    Public Sub LoadParameters(ByRef rCmd As Data.Common.DbCommand)

        For Each tItem As SQLQueryItem In QItems
            'tItem.LoadParam(rCmd)
        Next

    End Sub




End Class


''' <summary>
''' Represents a query item in an sql string
''' </summary>
''' <remarks></remarks>
Public Class SQLQueryItem


    ''' <summary>
    ''' The format to use when converting date objects to string for use in SQL statements.
    ''' </summary>
    Public Const cFDate As String = "{0:MM-dd-yyyy HH:mm:ss}"


#Region " Member Variables "

    Private m_DisplayFieldName As String = ""
    Private m_SQLFieldName As String = ""
    Private m_SQLOperator As eOperators

    Private m_DbType As Integer

    Private m_Value1 As String = ""
    Private m_Value2 As String = ""

    Private m_Combiner As eCatters


    Public Const cOpLike = "LIKE"
    Public Const cOPEquals = "EQUALS"
    Public Const cOPBetween = "BETWEEN"
    Public Const cWildCard = "Wild"

#End Region

    Public Enum eOperators As Byte
        OEquals
        OBetween
        OLike
        OGreater
        OLess
        OWild

    End Enum


    Public Enum eCatters As Byte
        CAnd
        COr
    End Enum


#Region " Properties "

    ''' <summary>
    ''' The first parameter name
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ParamNameValue1() As String
        Get
            Return "p1" & m_SQLFieldName
        End Get
    End Property

    ''' <summary>
    ''' The 2nd paramter name, if there is one.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ParamNameValue2() As String
        Get
            Return "p2" & Me.m_SQLFieldName
        End Get
    End Property

    Public Property Value1() As String
        Get
            Return Me.m_Value1
        End Get
        Set(ByVal value As String)
            Me.m_Value1 = value
        End Set
    End Property

    Public Property Value2() As String
        Get
            Return Me.m_Value2
        End Get
        Set(ByVal value As String)
            Me.m_Value2 = Value2
        End Set
    End Property

    ''' <summary>
    ''' This is the combineer (and, or)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Combiner() As eCatters
        Get
            Return Me.m_Combiner
        End Get
        Set(ByVal value As eCatters)
            Me.m_Combiner = value
        End Set
    End Property

    ''' <summary>
    ''' The sql friendly version of the (AND or) combiners
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property CombinerStr() As String '
        Get
            Return Me.m_Combiner.ToString.Substring(1)
        End Get
    End Property

    ''' <summary>
    ''' The display name is used
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shadows ReadOnly Property ToString() As String
        Get
            Return Me.m_DisplayFieldName
        End Get

    End Property

    Public Property DisplayFieldName() As String
        Get
            Return Me.m_DisplayFieldName
        End Get
        Set(ByVal value As String)
            Me.m_DisplayFieldName = value
        End Set
    End Property

#End Region


#Region " Constructors "


    Public Sub New(ByVal vDisplayName As String, ByVal vFieldName As String, ByVal vDataType As DbType, ByVal vOperator As eOperators, ByVal vValue1 As String, ByVal vCombiner As eCatters)


        Me.m_DisplayFieldName = vDisplayName
        Me.m_SQLFieldName = vFieldName
        Me.m_DbType = vDataType
        Me.m_SQLOperator = vOperator
        Me.m_Value1 = vValue1
        Me.m_Combiner = vCombiner


    End Sub

    Public Sub New(ByVal vDisplayName As String, ByVal vFieldName As String, ByVal vDataType As DbType, ByVal vOperator As eOperators, ByVal vValue1 As String, ByVal vValue2 As String, ByVal vCombiner As eCatters)

        Me.New(vDisplayName, vFieldName, vDataType, vOperator, vValue1, vCombiner)

        Me.m_Value2 = vValue2

    End Sub

#End Region


#Region " SQL Ouput "

    Public Function SQLOut() As String

        Dim tSQL As String = Me.m_SQLFieldName




        Select Case m_SQLOperator

            Case eOperators.OWild
                'Need to determine if wildcards are in it


                If Me.Value1.StartsWith("<>") Then

                    'Checking if there are wildcards
                    If Me.m_DbType = Data.DbType.String Then
                        'gotta check for wildcards

                        If Me.Value1.IndexOf("*") <> -1 Then
                            'wilds
                            tSQL = tSQL & " NOT LIKE " & Me.SQLPrep

                        Else
                            'No wilds
                            tSQL = tSQL & "<>" & Me.SQLPrep

                        End If

                    Else
                        tSQL = tSQL & "<>" & Me.SQLPrep
                    End If


                    Return tSQL

                ElseIf Me.Value1.StartsWith("*") Or Me.Value1.EndsWith("*") Then
                    'Like clause must be a string
                    tSQL = tSQL & " LIKE " & Me.SQLPrep


                    Return tSQL


                ElseIf Me.Value1.StartsWith("=") Then

                    tSQL = tSQL & "=" & Me.SQLPrep

                    Return tSQL




                ElseIf Me.Value1.StartsWith(">") Then

                    'Me.m_Value1 = Me.Value1.Substring(1) 'stripping off the =



                    tSQL = tSQL & ">" & Me.SQLPrep

                    Return tSQL
                    'If Me.m_DbType = Data.DbType.Int32 Or Me.m_DbType = Data.DbType.Decimal Then
                    '    'Number or date
                    '    Return Me.m_SQLFieldName & ">@" & Me.ParamNameValue1

                    'ElseIf Me.m_DbType = Data.DbType.Date Then
                    '    Return Me.m_SQLFieldName & ">@" & Me.ParamNameValue1
                    'End If

                ElseIf Me.Value1.StartsWith("<") Then



                Else
                    Throw New Exception("Operator Not Supported " & Me.Value1)

                End If

            Case eOperators.OLike

                'Couldn't get the like to work parametized so doing it this way
                Return Me.m_SQLFieldName & " LIKE '%" & Me.Value1 & "%'"

            Case eOperators.OBetween
                'Using both

                Return Me.m_SQLFieldName & " BETWEEN @" & Me.ParamNameValue1 & " AND @" & Me.ParamNameValue2

            Case eOperators.OEquals

                Return Me.m_SQLFieldName & "=@" & Me.ParamNameValue1

            Case Else
                Throw New Exception("Not support")

        End Select


    End Function


    'Public Sub LoadParam(ByRef rCmd As Data.Common.DbCommand)


    '    If Me.m_SQLOperator <> eOperators.OLike Then

    '        If Me.m_DbType = Data.DbType.Int32 Then
    '            DBSaber.StdTools.CommandParamAdd(rCmd, Me.ParamNameValue1, DbType.Int32, Convert.ToInt32(Me.m_Value1))

    '        Else
    '            DBSaber.StdTools.CommandParamAdd(rCmd, Me.ParamNameValue1, Me.m_DbType, Me.m_Value1)

    '        End If
    '    End If

    '    If Me.m_SQLOperator = eOperators.OBetween Then
    '        'have to do the next one
    '        DBSaber.StdTools.CommandParamAdd(rCmd, Me.ParamNameValue2, Me.m_DbType, Me.m_Value2)

    '    End If

    'End Sub

#End Region



    ''' <summary>
    ''' Creats a collection full of operators ("=, like") for a particular data type
    ''' </summary>
    ''' <param name="vDataType"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Opts(ByVal vDataType As Data.DbType) As Collection

        Dim tCollect As New Collection


        Select Case vDataType

            Case DbType.String
                tCollect.Add(SQLQueryItem.eOperators.OEquals.ToString.Substring(1))
                tCollect.Add(SQLQueryItem.eOperators.OLike.ToString.Substring(1))

            Case DbType.DateTime
                tCollect.Add(SQLQueryItem.eOperators.OBetween.ToString.Substring(1))
                tCollect.Add(SQLQueryItem.eOperators.OGreater.ToString.Substring(1))
                tCollect.Add(SQLQueryItem.eOperators.OLess.ToString.Substring(1))

            Case DbType.Int16, DbType.Int32

                tCollect.Add(SQLQueryItem.eOperators.OBetween.ToString.Substring(1))
                tCollect.Add(SQLQueryItem.eOperators.OGreater.ToString.Substring(1))
                tCollect.Add(SQLQueryItem.eOperators.OLess.ToString.Substring(1))

            Case Else

                Throw New Exception("Data Type: " & vDataType & " not support")


        End Select


        Return tCollect

    End Function





#Region " SQLValue Functions "

    ''' <summary>
    ''' Fixes strings for use in SQL statements that might have invalid characters
    ''' </summary>
    Public Shared Function SQLStrClean(ByVal vStr As String) As String

        Return vStr.Replace("'", "''")

    End Function

    ''' <summary>
    ''' Creates an SQL Friendly value of the String data type. 
    ''' String have to be escaped. 
    ''' Usage: FieldName=" SQLValue
    ''' </summary>
    Public Shared Function SQLValue(ByVal value As String) As String
        'Need to clean up the strings. Helps prevent SQL injection attacks
        Return "'" & SQLStrClean(value) & "'"
    End Function



#End Region


    ''' <summary>
    ''' Does alot of sql prepping for the sql value
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function SQLPrep() As String

        Dim tValue As String = Me.OpStrip(Me.Value1)

        If Me.m_DbType = Data.DbType.String Then

            'access through ado has the wildcard thing
            Return SQLValue(SQLStrClean(tValue).Replace("*", "%"))

        ElseIf Me.m_DbType = Data.DbType.DateTime Or Me.m_DbType = Data.DbType.Date Then
            Return SQLValue(Convert.ToDateTime(tValue))
        Else
            'Number so don't really need to do anything
            Return tValue

        End If

    End Function


    ''' <summary>
    ''' Determines if a value is valid for the datatype
    ''' </summary>
    ''' <param name="vValue"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ValueValid(ByVal vValue As String) As Boolean


        Dim tValue As String = Me.OpStrip(vValue)

        If Me.m_DbType = Data.DbType.String Then
            'Everything is valid for strings
            Return True

        ElseIf Me.m_DbType = Data.DbType.Date Or Me.m_DbType = Data.DbType.DateTime Then
            'Not allowing wildcards

            Dim tDate As Date

            Return Date.TryParse(tValue, tDate)



            'Checking for a valid date


        Else

            'Checking for a valid number

            Dim tNum As Integer

            Return Integer.TryParse(tValue, tNum)

        End If



    End Function

    ''' <summary>
    ''' Removes the operator from the value
    ''' </summary>
    ''' <param name="vStr"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function OpStrip(ByVal vStr As String) As String

        If vStr.StartsWith("<>") Then
            Return vStr.Substring(2)
        ElseIf vStr.StartsWith("=") Or vStr.StartsWith(">") Or vStr.StartsWith("<") Then
            'Note: Include the operator * because the query value contains that op
            Return vStr.Substring(1)
        Else
            Return vStr
        End If


    End Function

    ''' <summary>
    ''' Determines if the value contains a valid operator for the data type
    ''' </summary>
    ''' <param name="vValue"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function OpValid(ByVal vValue As String) As Boolean

        If Me.m_DbType = Data.DbType.String Then
            'Not allowing greater than


            If (vValue.StartsWith(">") Or vValue.StartsWith("<")) And vValue.StartsWith("<>") = False Then
                Return False
            Else
                Return True
            End If


        ElseIf Me.m_DbType = Data.DbType.Date Or Me.m_DbType = Data.DbType.DateTime Then
            'Not allowing wildcards
            If vValue.StartsWith("*") Or vValue.EndsWith("*") Then
                Return False
            Else
                Return True
            End If

            'Checking for a valid date


        Else
            'Not allowing wildcards
            If vValue.StartsWith("*") Or vValue.EndsWith("*") Then
                Return False
            Else
                Return True
            End If

            'Checking for a valid number

        End If


    End Function

    ''' <summary>
    ''' Determines what operator there is for a speci
    ''' </summary>
    ''' <param name="vValue"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function OpHas(ByVal vValue As String) As Boolean

        If Me.m_DbType = Data.DbType.String Then

            If vValue.StartsWith("*") = False And vValue.EndsWith("*") = False And vValue.StartsWith("=") = False And vValue.StartsWith("<>") = False Then
                'They didn't have any operator
                Return False
            Else
                Return True
            End If

        ElseIf Me.m_DbType = Data.DbType.Date Or Me.m_DbType = Data.DbType.DateTime Then

            If vValue.StartsWith("<") = False And vValue.StartsWith(">") = False And vValue.StartsWith("=") = False And vValue.StartsWith("<>") = False Then
                Return False
            Else
                Return True
            End If

        Else
            'Number
            If vValue.StartsWith("<") = False And vValue.StartsWith(">") = False And vValue.StartsWith("=") = False Then
                Return False
            Else
                Return True
            End If

        End If


    End Function

    ''' <summary>
    ''' Giving them an operator incase they don't have one. Defaulting to an op for the data type
    ''' </summary>
    ''' <param name="vValue"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function OpPreProcess(ByVal vValue As String) As String

        If Me.m_DbType = Data.DbType.String Then
            'If they don't have an op so wildcard both sides

            If Me.OpHas(vValue) = False Then
                Return "*" & vValue & "*"
            Else
                Return vValue
            End If

        ElseIf Me.m_DbType = Data.DbType.Date Or Me.m_DbType = Data.DbType.DateTime Then

            'Date: don't have op so =

            If Me.OpHas(vValue) = False Then
                Return "=" & vValue
            Else
                Return vValue
            End If


        Else
            'Number so =
            If Me.OpHas(vValue) = False Then
                Return "=" & vValue
            Else
                Return vValue
            End If

        End If


    End Function

End Class

